using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Reflection;
using RainbowLibraryCE.Common;
using RainbowLibraryCE.Misc;
using RainbowLibraryCE.Network;

namespace PocketRassegnaRAI
{
    public partial class frmMain : Form
    {
        private CFileDownloader _objDownloader;
        private string _strFileToDownloadPath;
        private Label _lblFeedback;

        public frmMain()
        {
            InitializeComponent();

            Cursor.Current = Cursors.WaitCursor;

            GlobalVars.ProgPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase) + "\\";
            GlobalVars.SettingFilePath = GlobalVars.ProgPath + Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().GetName().CodeBase) + ".xml";

            //carica le impostazioni dal file xml
            loadSettings();

            //carica i form per ottimizzarne la gestione
            GlobalVars.Forms.Add("frmMain", this);
            GlobalVars.Forms.Add("frmSettings", new frmSettings());
            GlobalVars.Forms.Add("frmAbout", new frmAbout());
            //GlobalVars.Forms.Add("frmFileSelect", new frmFileSelect());

            //popola i combobox con i valori
            cboxLevel1.DataSource = GlobalVars.Settings.Level1Items;
            cboxLevel1.ValueMember = "ID";
            cboxLevel1.DisplayMember = "Desc";

            //popola i combobox con i valori
            cboxLevel2.DataSource = GlobalVars.Settings.Level2Items;
            cboxLevel2.ValueMember = "ID";
            cboxLevel2.DisplayMember = "Desc";

            //aggiustamenti grafici vari
            panelDownloadStatus.Top = 20;
            panelDownloadStatus.Left = 20;
            panelDownloadStatus.Visible = false;

            //imposta un eventuale file da leggere
            chooseFileToDownload();

            Cursor.Current = Cursors.Default;
        }

        #region File download delegates

        //metodo che viene chiamato quando inizia il download di un file
        private void FileDownloader_DownloadStarting(CFileDownloader objThread)
        {
            SetStatus("Inizializzazione...", 0);
        }

        //metodo che viene chiamato quando il download di un file e' in corso
        private void FileDownloader_DownloadInProgress(CFileDownloader objThread)
        {
            int intValue;

            if (objThread.FileSize > 0)
                intValue = Convert.ToInt32(objThread.FileDownloadedSize / Convert.ToDecimal(objThread.FileSize) * 100);
            else
                intValue = 0;
            SetStatus(objThread.FileDownloadedSize / 1024 + " di " + objThread.FileSize / 1024 + "Kb", intValue);
        }

        //metodo che viene chiamato quando termina il download di un file
        private void FileDownloader_DownloadCompleted(CFileDownloader objThread, bool booIsSuccess, string strErrorMessage)
        {
            PowerManager.EnableSuspend();

            if (booIsSuccess)
                SetStatus("Terminato...", 100);
            else
            {
                if (File.Exists(objThread.FilePath)) File.Delete(objThread.FilePath);
                if (String.Empty != strErrorMessage) MessageBox.Show("Problemi durante il download:\n" + strErrorMessage);
                SetStatus(String.Empty, -1);
            }
        }

        //necessaria per aggiornare senza intoppi l'interfaccia
        delegate void delSetStatus(string strStatus, int intPercentComplete);
        private void SetStatus(string strStatus, int intPercentComplete)
        {
            if (lblDownloadStatus.InvokeRequired)
            {
                delSetStatus s = new delSetStatus(SetStatus);
                this.Invoke(s, new Object[] { strStatus, intPercentComplete });
            }
            else
            {
                lblDownloadStatus.Text = strStatus;
                //download in corso
                if (intPercentComplete > 0)
                {
                    pbarDownloading.Value = intPercentComplete;
                }
                //download terminato ma fallito
                if (-1 == intPercentComplete)
                {
                    setDownloadInterface(false);
                    chooseFileToDownload();
                }
                //download terminato con successo
                if (100 == intPercentComplete)
                {
                    setDownloadInterface(false);
                    chooseFileToDownload();
                }
            }
        }

        #endregion

        #region Events

        private void menuItemSettings_Click(object sender, EventArgs e)
        {
            frmSettings objForm;

            objForm = (frmSettings)GlobalVars.Forms["frmSettings"];
            objForm.LoadSettings();
            objForm.ShowDialog();
            chooseFileToDownload();  //aggiorna lo stato dei bottoni
        }

        private void menuItemAbout_Click(object sender, EventArgs e)
        {
            ((frmAbout)GlobalVars.Forms["frmAbout"]).ShowDialog();
        }

        private void menuItemClose_Click(object sender, EventArgs e)
        {
            if (DialogResult.OK == MessageBox.Show("Chiudere l'applicazione?", GlobalVars.ProgName, MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1)) Application.Exit();
        }

        //lancia lo scarico del file della rassegna indicata
        private void pboxDownload_Click(object sender, EventArgs e)
        {
            ThreadStart tsDelegate;
            Thread objThread;
            string strURL;
            string strFilePath;


            Cursor.Current = Cursors.WaitCursor;
            giveVisualFeedback(lblDownload);

            //inizia il download del file
            setDownloadInterface(true);

            strURL = GlobalVars.Settings.BaseURL + _strFileToDownloadPath;
            strFilePath = GlobalVars.Settings.CachePath + _strFileToDownloadPath.Replace("/", "\\");

            if (!InternetConnection.IsPresent())
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Non si e' connessi ad internet, impossibile scaricare il file richiesto", GlobalVars.ProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                setDownloadInterface(false);
                return;
            }

            Directory.CreateDirectory(Path.GetDirectoryName(strFilePath));
            _objDownloader = new CFileDownloader(strURL, strFilePath);
            _objDownloader.DownloadStarting += new CFileDownloader.delDownloadStarting(FileDownloader_DownloadStarting);
            _objDownloader.DownloadInProgress += new CFileDownloader.delDownloadInProgress(FileDownloader_DownloadInProgress);
            _objDownloader.DownloadCompleted += new CFileDownloader.delDownloadCompleted(FileDownloader_DownloadCompleted);
            tsDelegate = new ThreadStart(_objDownloader.StartDownload);
            objThread = new Thread(tsDelegate);
            objThread.Name = "Download Thread";
            objThread.Start();
            PowerManager.DisableSuspend();
            Cursor.Current = Cursors.Default;
        }

        //apre il file selezionato con il programma apposito
        private void pboxOpen_Click(object sender, EventArgs e)
        {
            giveVisualFeedback(lblOpen);

            if (String.Empty == GlobalVars.Settings.PDFAppPath)
                MessageBox.Show("Nessun programma impostato per visualizzare i PDF!\nAndare nelle impostazioni per selezionarne uno.", GlobalVars.ProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            else
                try
                {
                    ProcessHelp.LaunchExec(GlobalVars.Settings.PDFAppPath, "\"" + strGetCacheFilePath() + "\"");
                }
                catch (Exception)
                {
                    MessageBox.Show("Impossibile aprire il programma per visualizzare i PDF!\nAndare nelle impostazioni e modificarlo.", GlobalVars.ProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
        }

        //cancella il file attualmente selezionato se e' presente in cache
        private void pboxDelete_Click(object sender, EventArgs e)
        {
            giveVisualFeedback(lblDelete);

            if (DialogResult.Yes == MessageBox.Show("Cancellare il file presente nella cache?", GlobalVars.ProgName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2))
            {
                string strMessage = String.Empty;

                try
                {
                    File.Delete(strGetCacheFilePath());
                    strMessage = "File eliminato";
                }
                catch (Exception objEx)
                {
                    strMessage = "Problemi nell'eliminazione del file:\n" + objEx.Message;
                }
                finally
                {
                    MessageBox.Show(strMessage, GlobalVars.ProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    chooseFileToDownload(); //aggiorna i controlli disponibili
                }
            }
        }

        //controlla se e' disponibile una nuova versione del programma
        private void timCheckUpdate_Tick(object sender, EventArgs e)
        {
            int intLatestVersion;

            try
            {
                intLatestVersion = CheckSwVersion.GetLatestVersion("http://www.rainbowbreeze.it/devel/getlatestversion.php", GlobalVars.Settings.ProgUniqueID, GlobalVars.ProgID, GlobalVars.ProgVersion);
                if (0 < intLatestVersion)
                {
                    timCheckUpdate.Enabled = false;
                    if (intLatestVersion > GlobalVars.ProgVersion)
                        MessageBox.Show("E' disponibile una nuova versione del programma.\n  Versione attuale: " + GlobalVars.ProgVersion + "\n  Nuova versione: " + intLatestVersion, GlobalVars.ProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
            }
            catch (Exception)
            {
                timCheckUpdate.Enabled = false;
            }
        }

        private void cboxLevel1_SelectedIndexChanged(object sender, EventArgs e)
        {
            chooseFileToDownload();
        }

        private void cboxLevel2_SelectedIndexChanged(object sender, EventArgs e)
        {
            chooseFileToDownload();
        }

        private void pboxCancelDownload_Click(object sender, EventArgs e)
        {
            if (null != _objDownloader)
            {
                _objDownloader.CancelDownload();
            }
        }

        #endregion

        #region Draw methods

        private void frmMain_Paint(object sender, PaintEventArgs e)
        {
            SolidBrush objBrush;
            Rectangle objRect;

            //disegna un'ombra sotto i combobox
            objBrush = new SolidBrush(Color.Gray);
            objRect = new Rectangle(cboxLevel1.Left + 2, cboxLevel1.Top + 2, cboxLevel1.Width, cboxLevel1.Height);
            e.Graphics.FillRectangle(objBrush, objRect);
            objRect = new Rectangle(cboxLevel2.Left + 2, cboxLevel2.Top + 2, cboxLevel2.Width, cboxLevel2.Height);
            e.Graphics.FillRectangle(objBrush, objRect);
            objBrush.Dispose();
        }

        private void panelDownloadStatus_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawRectangle(new Pen(SystemColors.WindowFrame, 2), 1, 1, panelDownloadStatus.Width - 3, panelDownloadStatus.Height - 3);
            e.Graphics.FillRectangle(new SolidBrush(Color.Gray), new Rectangle(pbarDownloading.Left + 2, pbarDownloading.Top + 2, pbarDownloading.Width, pbarDownloading.Height));
            //base.OnPaint(e);
        }

        #endregion

        #region Private methods

        //carica le impostazioni dal file dei settaggi, se non esiste lo crea
        private void loadSettings()
        {
            CRainbowAppSettings<CSettings> objSettingsHandler;

            objSettingsHandler = new CRainbowAppSettings<CSettings>(GlobalVars.SettingFilePath);

            if (File.Exists(GlobalVars.SettingFilePath))
            {
                //carica il file con le impstazioni
                objSettingsHandler.LoadSettings(GlobalVars.SettingFilePath);
            }
            else
            {
                //altrimenti lo crea
                objSettingsHandler.Settings = new CSettings();

                //carica le impostazioni del programma
                objSettingsHandler.Settings.Level1Title = "Seleziona la rassegna stampa:";
                objSettingsHandler.Settings.Level1Items.Add(new CComboData(0, "RaiCinema"));
                objSettingsHandler.Settings.Level1Items.Add(new CComboData(1, "RaiNet"));
                objSettingsHandler.Settings.Level1Items.Add(new CComboData(2, "RaiSat"));
                objSettingsHandler.Settings.Level1Items.Add(new CComboData(3, "RaiWay"));
                objSettingsHandler.Settings.Level1Items.Add(new CComboData(4, "Sipra"));

                objSettingsHandler.Settings.Level2Title = "Seleziona il giorno:";
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(0, "-Odierna-"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(1, "Lunedi'"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(2, "Martedi'"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(3, "Mercoledi'"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(4, "Giovedi'"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(5, "Venerdi'"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(6, "Sabato"));
                objSettingsHandler.Settings.Level2Items.Add(new CComboData(7, "Domenica"));

                objSettingsHandler.Settings.FilesToDownload.Add("0,0,raicinema/cin_odierna.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,1,raicinema/cin_lunedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,2,raicinema/cin_martedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,3,raicinema/cin_mercoledi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,4,raicinema/cin_giovedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,5,raicinema/cin_venerdi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,6,raicinema/cin_sabato.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("0,7,raicinema/cin_domenica.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,0,rainet/net_odierna.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,1,rainet/net_lunedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,2,rainet/net_martedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,3,rainet/net_mercoledi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,4,rainet/net_giovedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,5,rainet/net_venerdi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,6,rainet/net_sabato.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("1,7,rainet/net_domenica.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,0,raisat/sat_odierna.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,1,raisat/sat_lunedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,2,raisat/sat_martedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,3,raisat/sat_mercoledi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,4,raisat/sat_giovedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,5,raisat/sat_venerdi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,6,raisat/sat_sabato.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("2,7,raisat/sat_domenica.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,0,raiway/way_odierna.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,1,raiway/way_lunedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,2,raiway/way_martedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,3,raiway/way_mercoledi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,4,raiway/way_giovedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,5,raiway/way_venerdi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,6,raiway/way_sabato.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("3,7,raiway/way_domenica.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,0,sipra/sipra_odierna.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,1,sipra/sipra_lunedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,2,sipra/sipra_martedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,3,sipra/sipra_mercoledi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,4,sipra/sipra_giovedi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,5,sipra/sipra_venerdi.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,6,sipra/sipra_sabato.pdf");
                objSettingsHandler.Settings.FilesToDownload.Add("4,7,sipra/sipra_domenica.pdf");

                objSettingsHandler.Settings.SeparatorChar = ",";

                objSettingsHandler.Settings.BaseURL = @"http://www.ufficiostampa.rai.it/aree/";
                objSettingsHandler.Settings.CachePath = GlobalVars.ProgPath + "cache\\";
                objSettingsHandler.Settings.ProgUniqueID = DateTime.Now.ToString("yyMMddHHmmss");
                objSettingsHandler.Settings.PDFAppPath = RegistryHelper.GetDefaultApplication(".pdf");

                objSettingsHandler.SaveSettings();
            }
            GlobalVars.Settings = objSettingsHandler.Settings;
        }

        private void chooseFileToDownload()
        {
            if (cboxLevel1.SelectedIndex > -1 && cboxLevel2.SelectedIndex > -1)
            {
                _strFileToDownloadPath = GlobalVars.FilesToDownload[((CComboData)(cboxLevel1.SelectedItem)).ID, ((CComboData)(cboxLevel2.SelectedItem)).ID];
                pboxDownload.Enabled = true;
                lblDownload.Enabled = true;
                setInterfaceReadyToShow(File.Exists(GlobalVars.Settings.CachePath + _strFileToDownloadPath));
            }
            else
            {
                pboxDownload.Enabled = false;
                lblDownload.Enabled = false;
                setInterfaceReadyToShow(false);
            }
        }

        private void setInterfaceReadyToShow(bool booReady)
        {
            pboxOpen.Visible = booReady;
            lblOpen.Visible = booReady;
            pboxDelete.Visible = booReady;
            lblDelete.Visible = booReady;
            if (booReady)
                lblLastData.Text = File.GetLastWriteTime(GlobalVars.Settings.CachePath + _strFileToDownloadPath.Replace("/", "\\")).ToShortDateString();
            else
                lblLastData.Text = "---";
        }

        //imposta i controlli in modalita' download o non download (normale)
        private void setDownloadInterface(bool booDownloading)
        {
            if (booDownloading)
            {
                panelDownloadStatus.Location = new Point((Screen.PrimaryScreen.WorkingArea.Width - panelDownloadStatus.Width) / 2, (Screen.PrimaryScreen.WorkingArea.Height - panelDownloadStatus.Height) / 4);
                panelDownloadStatus.Visible = true;
                pbarDownloading.Value = 0;
                //disabilita tutti i controlli
                lblTitleLevel1.Enabled = false;
                lblTitleLevel2.Enabled = false;
                lblTitleDisp.Enabled = false;
                lblDownload.Enabled = false;
                lblOpen.Enabled = false;
                lblDelete.Enabled = false;
                lblLastData.Enabled = false;
                cboxLevel2.Enabled = false;
                cboxLevel1.Enabled = false;
                pboxDownload.Enabled = false;
                pboxOpen.Enabled = false;
                pboxDelete.Enabled = false;
                menuItemFile.Enabled = false;
                menuItemClose.Enabled = false;
            }
            else
            {
                panelDownloadStatus.Visible = false;
                //riabilita tutti i controlli
                lblTitleLevel1.Enabled = true;
                lblTitleLevel2.Enabled = true;
                lblTitleDisp.Enabled = true;
                lblDownload.Enabled = true;
                lblOpen.Enabled = true;
                lblDelete.Enabled = true;
                lblLastData.Enabled = true;
                cboxLevel2.Enabled = true;
                cboxLevel1.Enabled = true;
                pboxDownload.Enabled = true;
                pboxOpen.Enabled = true;
                pboxDelete.Enabled = true;
                menuItemFile.Enabled = true;
                menuItemClose.Enabled = true;
            }
        }

        private void giveVisualFeedback(Label lblFeedback)
        {
            Color objSwap;

            _lblFeedback = lblFeedback;
            objSwap = _lblFeedback.ForeColor;
            _lblFeedback.ForeColor = _lblFeedback.BackColor;
            _lblFeedback.BackColor = objSwap;
            timFeedback.Enabled = true;
            Application.DoEvents();
        }
        
        private string strGetCacheFilePath()
        {
            return GlobalVars.Settings.CachePath + _strFileToDownloadPath.Replace("/", "\\");
        }

        private void timFeedback_Tick(object sender, EventArgs e)
        {
            Color objSwap;

            timFeedback.Enabled = false;
            objSwap = _lblFeedback.ForeColor;
            _lblFeedback.ForeColor = _lblFeedback.BackColor;
            _lblFeedback.BackColor = objSwap;
        }



        #endregion

    }
}